/*******************************************************************************
 * Copyright (c) 2005 The Regents of the University of California. 
 * This material was produced under U.S. Government contract W-7405-ENG-36 
 * for Los Alamos National Laboratory, which is operated by the University 
 * of California for the U.S. Department of Energy. The U.S. Government has 
 * rights to use, reproduce, and distribute this software. NEITHER THE 
 * GOVERNMENT NOR THE UNIVERSITY MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR 
 * ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE. If software is modified 
 * to produce derivative works, such modified software should be clearly marked, 
 * so as not to confuse it with the version available from LANL.
 * 
 * Additionally, this program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * LA-CC 04-115
 *******************************************************************************/
package org.eclipse.ptp.tau.launchtau.internal;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.text.MessageFormat;
import java.util.Calendar;
import java.util.TimeZone;

import org.eclipse.cdt.core.CCorePlugin;
import org.eclipse.cdt.core.model.ICProject;
import org.eclipse.cdt.make.core.IMakeTarget;
import org.eclipse.cdt.make.internal.core.MakeTargetManager;
import org.eclipse.cdt.managedbuilder.core.IConfiguration;
import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
import org.eclipse.cdt.managedbuilder.core.IManagedProject;
import org.eclipse.cdt.managedbuilder.core.ITool;
import org.eclipse.cdt.managedbuilder.core.IToolChain;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ptp.core.IPTPLaunchConfigurationConstants;
import org.eclipse.ptp.core.PTPCorePlugin;
import org.eclipse.ptp.launch.internal.ParallelLaunchConfigurationDelegate;
import org.eclipse.ptp.tau.launchtau.ui.ITAULaunchConfigurationConstants;
import org.eclipse.ptp.tau.options.TAUOptionsPlugin;
import org.eclipse.ptp.tau.perfdmf.PerfDMFUIPlugin;
import org.eclipse.swt.widgets.Display;


/**
 * 
 */
public class TAULaunchConfigurationDelegate extends ParallelLaunchConfigurationDelegate {
	
	private IManagedProject managedBuildProj=null;
	private IConfiguration newBuildConfig=null;
	private IConfiguration olddefbuildconf=null;
	private ICProject thisCProject=null;
	private IProject thisProject = null;
	
	private String tbpath = null;
	private String tmakepath = null;
	private String tauoptchunk = null;
	private String application = null;
	
	private boolean managed = false;
	
	
	private void initTauCompilers(ILaunchConfiguration configuration) throws CoreException
	{
		//Build the tau compiler run string based on the TAU preferences
		IPreferenceStore pstore = TAUOptionsPlugin.getDefault().getPreferenceStore();
		String archpath=pstore.getString("TAUCDTArchPath");
		tbpath = archpath+File.separator+"bin";
		String tlpath = archpath+File.separator+"lib";
		
		tmakepath = "Makefile.tau-mpi-pdt";
		boolean makeargs[] = new boolean[7];
		makeargs[0]=configuration.getAttribute(ITAULaunchConfigurationConstants.CALLPATH, false);
		makeargs[1]=configuration.getAttribute(ITAULaunchConfigurationConstants.MEMORY, false);
		makeargs[2]=configuration.getAttribute(ITAULaunchConfigurationConstants.PAPI, false);
		makeargs[3]=configuration.getAttribute(ITAULaunchConfigurationConstants.PERF, false);
		makeargs[4]=configuration.getAttribute(ITAULaunchConfigurationConstants.TRACE, false);
		makeargs[5]=configuration.getAttribute(ITAULaunchConfigurationConstants.PHASE, false);
		
		String compiler = configuration.getAttribute(ITAULaunchConfigurationConstants.COMPILER, "");

		class makefilter implements FilenameFilter{
			boolean[] makeargs=null;
			String compiler=null;
			public makefilter(boolean[] input, String compiler){
				makeargs=input;
				this.compiler=compiler;
			}
			public boolean accept(File dir, String name) {
				if(name.indexOf("Makefile.tau")!=0 || name.indexOf("-pdt")<=0 || name.indexOf("-mpi")<=0 )return false;//|| name.indexOf("-icpc")<=0
				int namelength=20;//25
				
				if(makeargs[0]){
					if(name.indexOf("-callpath")<1)
						return false;
					namelength+=9;
				}
				else
				if(name.indexOf("-callpath")>-1)
					return false;
				
				if(makeargs[1]){
					if(name.indexOf("-memory")<1)
						return false;
					namelength+=7;
				}
				else
				if(name.indexOf("-memory")>-1)
					return false;	
				
				if(makeargs[2]){
					if(name.indexOf("-papi")<1)
						return false;
					namelength+=5;
				}
				else
					if(name.indexOf("-papi")>-1)
						return false;	
				
				if(makeargs[3]){
					if(name.indexOf("-perf")<1)
						return false;
					namelength+=5;
				}
				else
					if(name.indexOf("-perf")>-1)
						return false;
				
				if(makeargs[4]){
					if(name.indexOf("-trace")<1 || name.indexOf("-profile")<1)
						return false;
					namelength+=6;
					namelength+=8;
				}
				else
					if(name.indexOf("-trace")>-1)
						return false;
				
				if(makeargs[5]){
					if(name.indexOf("-phase")<1)
						return false;
					namelength+=6;
				}
				else
					if(name.indexOf("-phase")>-1)
						return false;
				
				if(compiler.length()>0){
					if(name.indexOf("-"+compiler)<1)
						return false;
					namelength+=(compiler.length()+1);
				}
				
				if(name.length()!=namelength) return false;
				
				return true;
			}
		}

		File[] makefiles = null;
		File dir = new File(tlpath);

		if(dir.exists()){
			makefiles = dir.listFiles(new makefilter(makeargs,compiler));
		}

		if(makefiles.length>=1)
			tmakepath=makefiles[0].getName();
		else
			System.out.println("Could not find specified makefile!  Using options "+tmakepath);
		
		tmakepath=tlpath+File.separator+tmakepath;
		
		int selopt = configuration.getAttribute(ITAULaunchConfigurationConstants.SELECT, 0);
		String selcommand="";
		String selpath="";
		if(selopt==1)
		{
			selpath=thisCProject.getResource().getLocation().toOSString()+File.separator+"tau.selective";
			selcommand="-optTauSelectFile="+selpath;//+" -optKeepFiles";
		}
		else
		if(selopt==2)
		{
			selpath=configuration.getAttribute(ITAULaunchConfigurationConstants.SELECT_FILE, "");
			if(!selpath.equals(""))
				selcommand="-optTauSelectFile="+selpath;//+" -optKeepFiles"
		}
		
		tauoptchunk="";
		String tauopts = configuration.getAttribute(ITAULaunchConfigurationConstants.TAU_RUN_OPTS,"")+selcommand;//pstore.getString("TAUCDTOpts");
		if(!tauopts.equals("")){
			tauoptchunk=" -tau_options='"+tauopts+"'";
		}
	}
	
	
	private void standardMakeBuild(ILaunchConfiguration configuration) throws CoreException{
		
		File tauinc = new File(thisCProject.getResource().getLocation().toOSString()+File.separator+"eclipse.inc");
		File taudef = new File(thisCProject.getResource().getLocation().toOSString()+File.separator+"eclipse.inc.default");
		try{
			if(tauinc.exists())
			{
				InputStream in = new FileInputStream(tauinc);
				OutputStream out = new FileOutputStream(taudef);

				// Transfer bytes from in to out
				byte[] buf = new byte[1024];
				int len;
				while ((len = in.read(buf)) > 0) {
					out.write(buf, 0, len);
				}
				in.close();
				out.close();
			}

			BufferedWriter tauout = new BufferedWriter(new FileWriter(tauinc));

			tauout.write("ECLIPSE_CC=" +tbpath+File.separator+"tau_cc.sh" +" -tau_makefile="+tmakepath+tauoptchunk+"\n");
			tauout.write("ECLIPSE_CXX="+tbpath+File.separator+"tau_cxx.sh"+" -tau_makefile="+tmakepath+tauoptchunk+"\n");
			tauout.write("ECLIPSE_FC="+tbpath+File.separator+"tau_f90.sh"+" -tau_makefile="+tmakepath+tauoptchunk+"\n");
			tauout.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		//System.out.println(makefile.getContents().);
		//IMakeTarget selected = new IMakeTarget(); //= targetPart.getSelectedTarget();
		MakeTargetManager targetMan = new MakeTargetManager();
		targetMan.startup();
		//System.out.println(targetMan.hasTargetBuilder(thisProject));
		//targetMan.
		//String[] targbs = targetMan.getTargetBuilders(thisProject);
		//for(int i=0; i<targbs.length;i++)
		//	System.out.println(targbs[i]);

		IMakeTarget[] targs = targetMan.getTargets(thisProject);
		IMakeTarget select = null;
		//System.out.println(targs.length);
		for(int i=0;i<targs.length;i++)
		{
			if(targs[i].getName().equals("all"))
			{
				select = targs[i];
				break;
			}
			System.out.println(targs[i].getName()+" "+targs[i].getTargetBuilderID());
		}
		if(select==null)
		{
			System.out.println("No Make Target: all");
			return;}

		System.out.println(select.getBuildLocation());
		//System.out.println(select.);

		select.build(new NullProgressMonitor());
		//targbs[0].
		//IMakeTarget target = targetMan.createTarget(thisProject, "all", "all");

		//super.okPressed();
		//if (selected != null) {
		//TargetBuild.buildTargets(null, new IMakeTarget[] { select });//getParentShell()  
		//}
		targetMan.shutdown();
		//System.out.println("NOT MANAGED!!!");
		//return;
		if(taudef.exists())
		{
			InputStream in;
			try {
				in = new FileInputStream(taudef);

				OutputStream out = new FileOutputStream(tauinc);

				// Transfer bytes from in to out
				byte[] buf = new byte[1024];
				int len;
				while ((len = in.read(buf)) > 0) {
					out.write(buf, 0, len);
				}
				in.close();
				out.close();	
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	private void managedMakeBuild(ILaunchConfiguration configuration, IProgressMonitor monitor) throws CoreException
	{
		IManagedBuildInfo info = ManagedBuildManager.getBuildInfo(thisCProject.getResource());
		if (info == null||!info.isValid()){
			System.out.println("No info!!!");
			return;
		}
		//Get the managed project object
		managedBuildProj = info.getManagedProject();
		if (managedBuildProj == null){
			System.out.println("No managed project!!!");
			return;
		}
		olddefbuildconf=info.getDefaultConfiguration();//(IConfiguration) configlist.get(0);///chooseBuildConfiguration(configlist); ---FIX THIS---
		String binary = application.substring(application.indexOf(File.separator));
		String useconf= application.substring(0,application.indexOf(File.separator));
		//Make a list of the configurations already within the project
		IConfiguration[] buildconfigs = info.getManagedProject().getConfigurations();
		IConfiguration selectedconf = null;
		for(int i=0;i<buildconfigs.length;i++){
			if((buildconfigs[i].getName()).equals(useconf))
			{
				selectedconf=buildconfigs[i];
				break;
			}
		}
		
		if(selectedconf==null)
		{System.out.println("No Conf Selected");return;}
		if(selectedconf.getName()==null)
		{System.out.println("Uh Oh!");return;}
		
		//Make the new configuration name, and if there is already a configuration with that name, remove it.
		String taunameappend=("("+tmakepath.substring(tmakepath.lastIndexOf(".")+1)+")");
		String newname=selectedconf.getName();
		if(newname.indexOf(taunameappend)<0)
			newname += ("("+tmakepath.substring(tmakepath.lastIndexOf(".")+1)+")");
		
		IConfiguration[] confs = managedBuildProj.getConfigurations();
		for(int i =0; i<confs.length;i++)
		{
			if(confs[i].getName().equals(newname)||confs[i].getName().indexOf(newname)>=0)
			{
				managedBuildProj.removeConfiguration(confs[i].getId());
			}
		}
		//Make a copy of the selected configuration(Clone works, basic create does not) and rename it.
		newBuildConfig = managedBuildProj.createConfigurationClone(selectedconf, selectedconf.getId()+"."+ManagedBuildManager.getRandomNumber());
		if (newBuildConfig == null){
			System.out.println("No config!");
			//return;
		}
		newBuildConfig.setName(newname);
		IToolChain chain = newBuildConfig.getToolChain();
		ITool[] tools = chain.getTools();
		//Replace the compiler/linker commands with the correct tau compiler scripts and arguments.
		for(int i =0;i<tools.length;i++){
			String toolid=tools[i].getId();
			if(toolid.indexOf(".c.")>=0)
			{
				tools[i].setToolCommand(tbpath+File.separator+"tau_cc.sh"+" -tau_makefile="+tmakepath+tauoptchunk);
			}
			if(toolid.indexOf(".cpp.")>=0)
			{
				tools[i].setToolCommand(tbpath+File.separator+"tau_cxx.sh"+" -tau_makefile="+tmakepath+tauoptchunk);
			}
			if(toolid.indexOf(".fortran.")>=0)
			{
				tools[i].setToolCommand(tbpath+File.separator+"tau_f90.sh"+" -tau_makefile="+tmakepath+tauoptchunk);
			}
		}
		System.out.println(tbpath+File.separator+"tau_cc.sh"+" -tau_makefile="+tmakepath+tauoptchunk);
		ManagedBuildManager.saveBuildInfo(thisCProject.getProject(),true);
		//Build set the new configuration to default so we can build it.
		ManagedBuildManager.setDefaultConfiguration(thisCProject.getProject(),newBuildConfig);
		SubProgressMonitor buildmonitor = new SubProgressMonitor(monitor,10);
		//NullProgressMonitor nullmon = new NullProgressMonitor();
		try {
			thisProject.build(IncrementalProjectBuilder.FULL_BUILD, buildmonitor);//CLEAN_BUILD

		} catch (Exception e) {
			e.printStackTrace();
			return;
		}

		IFile programPath = thisProject.getFile(newname+binary);
		while(!programPath.exists() || !programPath.getLocation().toFile().exists())
		{
			//System.out.println("Looping Out!");
			if(monitor!=null&&buildmonitor.isCanceled())
			{
				ManagedBuildManager.setDefaultConfiguration(thisCProject.getProject(),olddefbuildconf);
				throw new OperationCanceledException();
				//abort(LaunchMessages.getResourceString("AbstractTAULaunchConfigurationDelegate.Application_file_does_not_exist"), new FileNotFoundException(LaunchMessages.getFormattedResourceString("AbstractTAULaunchConfigurationDelegate.Application_path_not_found", programPath.getLocation().toString())), IStatus.INFO);
			}
			long numMillisecondsToSleep = 1000; // 1 seconds
			try {
				Thread.sleep(numMillisecondsToSleep);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			programPath = thisProject.getFile(newname+binary);
		}
		ILaunchConfigurationWorkingCopy confWC = configuration.getWorkingCopy();
		confWC.setAttribute(IPTPLaunchConfigurationConstants.ATTR_APPLICATION_NAME, newname+binary);
		configuration = confWC.doSave();
	}
	
	private void cleanup(ILaunchConfiguration configuration) throws CoreException
	{
		if(managed)
		{
			ManagedBuildManager.setDefaultConfiguration(thisCProject.getProject(),olddefbuildconf);
			if(!configuration.getAttribute(ITAULaunchConfigurationConstants.NOCLEAN, false))
				managedBuildProj.removeConfiguration(newBuildConfig.getId());
		}
		ILaunchConfigurationWorkingCopy confWC = configuration.getWorkingCopy();
		confWC.setAttribute(IPTPLaunchConfigurationConstants.ATTR_APPLICATION_NAME, application);
		configuration = confWC.doSave();
	}
	
	public void launch(ILaunchConfiguration configuration, String mode, ILaunch launch, IProgressMonitor monitor) throws CoreException {
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}
		monitor.beginTask("",  250);
		monitor.setTaskName(MessageFormat.format("{0} . . .", new String[] { "Launching " + configuration.getName() }));
		if (monitor.isCanceled()) {
			return;
		}
		
		//Initialize variables used in all cases
		PTPCorePlugin.getDefault().getModelManager().refreshRuntimeSystems(new SubProgressMonitor(monitor, 50), false);
		thisProject = getProject(configuration);
		thisCProject = CCorePlugin.getDefault().getCoreModel().create(thisProject);
		initTauCompilers(configuration);
		application=configuration.getAttribute(IPTPLaunchConfigurationConstants.ATTR_APPLICATION_NAME, (String)null);

		//Rebuild project with TAU compiler scripts according to the project's nature
		if(!ManagedBuildManager.canGetBuildInfo(thisCProject.getResource()))
		{
			standardMakeBuild(configuration);
		}
		else
		{
			managed=true;
			managedMakeBuild(configuration,monitor);
		}
		
		//Launch the instrumented executable
		super.launch(configuration, mode, launch, monitor);
		if (monitor.isCanceled()) {
			cleanup(configuration);
			throw new OperationCanceledException();
		}
		
		//Collect data and clean up
		postlaunch(configuration,monitor);
		
	}
	
	private void postlaunch(ILaunchConfiguration configuration,IProgressMonitor monitor) throws CoreException{
		final String directory=thisCProject.getUnderlyingResource().getLocation().toOSString();
		final String projname = thisCProject.getElementName();
		final String projtype = tmakepath.substring(tmakepath.lastIndexOf("tau-")+4);

		class profilefilter implements FilenameFilter{
			public boolean accept(File dir, String name) {
				if(name.indexOf("profile.")!=0)
					return false;
				return true;
			}
		}
		
		class tracefilter implements FilenameFilter{
			public boolean accept(File dir, String name) {
				if(name.indexOf(".trc")>0||name.indexOf(".edf")>0)
					return true;
				return false;
			}
		}

		class perffilter implements FilenameFilter{
			public boolean accept(File dir, String name) {
				if(name.indexOf("perf_data.")!=0)
					return false;
				return true;
			}
		}

		File[] profiles = null;
		File dir = new File(directory);
		profilefilter profil=new profilefilter();
		profiles = dir.listFiles(profil);


		//perf_data.*
		//perf2tau
		boolean profilesdone = false;
		if(configuration.getAttribute(ITAULaunchConfigurationConstants.PERF, false))
		{
			perffilter seekdir = new perffilter();
			File[] perfdir = dir.listFiles(seekdir);
			while(perfdir==null || perfdir.length<1)
			{
				perfdir = dir.listFiles(seekdir);
				try {
					if(monitor.isCanceled())
					{
						cleanup(configuration);
						throw new OperationCanceledException();
					}
					//long numMillisecondsToSleep = 1000; // 1 seconds
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
			}

			String perf2tau = tbpath+File.separator+"perf2tau";
			String s = null;
			try {
				Process p = Runtime.getRuntime().exec(perf2tau, null, dir);
				int i = p.waitFor();
				if (i == 0)
				{
					BufferedReader stdInput = new BufferedReader(new InputStreamReader(p.getInputStream()));
					//read the output from the command
					while ((s = stdInput.readLine()) != null) 
					{
						System.out.println(s);
					}
				}
				else 
				{
					BufferedReader stdErr = new BufferedReader(new InputStreamReader(p.getErrorStream()));
					//read the output from the command
					while ((s = stdErr.readLine()) != null) 
					{
						System.out.println(s);
					}
				}
			}
			catch (Exception e) {System.out.println(e);}
			profilesdone = true;
		}

		int nprocs = (new Integer(getNumberOfProcesses(configuration))).intValue();
		while((profiles.equals(null) || profiles.length<nprocs)&&!profilesdone)
		{
			profiles = dir.listFiles(profil);
			try {
				if(monitor.isCanceled())
				{
					cleanup(configuration);
					throw new OperationCanceledException();
				}
				//long numMillisecondsToSleep = 1000; // 1 seconds
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		}

		cleanup(configuration);
		//Put the profile data in the database and delete any profile files
		Calendar cal = Calendar.getInstance(TimeZone.getDefault());
        String DATE_FORMAT = "yyyy-MM-dd_HH:mm:ss";
        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat(DATE_FORMAT);
        sdf.setTimeZone(TimeZone.getDefault());
        final String now = sdf.format(cal.getTime());
		
		
		final File[] remprofs = profiles;
		final boolean keepprofs=configuration.getAttribute(ITAULaunchConfigurationConstants.KEEPPROFS, false);
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				PerfDMFUIPlugin.addPerformanceData(projname, projtype, directory);
				if(!keepprofs)
					for(int i=0;i<remprofs.length;i++)
					{
						remprofs[i].delete();
					}
				else
				{
					

		            //trial.setName();
					File profdir = new File(directory+File.separator+"Profiles"+File.separator+projtype+File.separator+now);
					profdir.mkdirs();
					for(int i=0;i<remprofs.length;i++)
					{
						remprofs[i].renameTo(new File(profdir+File.separator+remprofs[i].getName()));
					}
				}

			}});
		/*
		if(configuration.getAttribute(ITAULaunchConfigurationConstants.TRACE, false))
		{
            //trial.setName();
			tracefilter tracefind = new tracefilter();
			File[] mvtrc=dir.listFiles(tracefind);
			while((mvtrc.equals(null) || mvtrc.length<nprocs*2))
			{
				mvtrc = dir.listFiles(tracefind);
				try {
					if(monitor.isCanceled())
					{
						//cleanup(configuration);
						throw new OperationCanceledException();
					}
					//long numMillisecondsToSleep = 1000; // 1 seconds
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
			}
			
			File trcdir = new File(directory+File.separator+"Traces"+File.separator+projtype+File.separator+now);
			trcdir.mkdirs();
			for(int i=0;i<mvtrc.length;i++)
			{
				mvtrc[i].renameTo(new File(trcdir+File.separator+remprofs[i].getName()));
			}
		}*/
	}
}
