/*******************************************************************************
 * Copyright (c) 2005 The Regents of the University of California. 
 * This material was produced under U.S. Government contract W-7405-ENG-36 
 * for Los Alamos National Laboratory, which is operated by the University 
 * of California for the U.S. Department of Energy. The U.S. Government has 
 * rights to use, reproduce, and distribute this software. NEITHER THE 
 * GOVERNMENT NOR THE UNIVERSITY MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR 
 * ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE. If software is modified 
 * to produce derivative works, such modified software should be clearly marked, 
 * so as not to confuse it with the version available from LANL.
 * 
 * Additionally, this program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * LA-CC 04-115
 *******************************************************************************/
package org.eclipse.ptp.tau.launchtau.ui;

import java.io.File;
import java.io.FilenameFilter;
import java.util.LinkedHashSet;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ptp.internal.core.CoreMessages;
import org.eclipse.ptp.launch.ui.PLaunchConfigurationTab;
import org.eclipse.ptp.tau.launchtau.internal.ui.LaunchImages;
import org.eclipse.ptp.tau.options.TAUOptionsPlugin;
//import org.eclipse.ptp.ui.PTPUIPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * 
 */
public class AnalysisTab extends PLaunchConfigurationTab {
	protected Button callpathCheck;
	protected Button memoryCheck;
	
	protected Button papiCheck;
	protected Button papiSelect;
	protected Composite papiComp;
	
	protected Button perfCheck;
	protected Button traceCheck;
	protected Button phaseCheck;
	//protected Button icpcCheck;
	protected Composite selComp;
	protected Label selectLabel;
	protected Button selectRadios[];
	protected Text compiler;
	
	protected Button nocleanCheck;
	protected Button keepprofsCheck;
	
	protected Text tauArch = null;
	protected Button browseSelfileButton = null;
	
	protected File[] makefiles=null;

	protected class WidgetListener extends SelectionAdapter implements ModifyListener,IPropertyChangeListener {
		public void widgetSelected(SelectionEvent e) {
			if(e.getSource() == browseSelfileButton) {
				handleSelfileBrowseButtonSelected();
			}
			if(e.getSource()==papiSelect){
				handlePapiSelect();
			}
			if(e.getSource().equals(selectRadios[2]))
			{
				if(!selectRadios[2].getSelection()){
					selComp.setEnabled(false);
					tauArch.setEnabled(false);
					tauArch.setEnabled(false);
				}
				else
				{
					selComp.setEnabled(true);
					tauArch.setEnabled(true);
					tauArch.setEnabled(true);
				}
			}
			
			updateLaunchConfigurationDialog();
		}

		public void propertyChange(PropertyChangeEvent event) {
			updateLaunchConfigurationDialog();
		}
		
		public void modifyText(ModifyEvent evt) {
			Object source = evt.getSource();
			if(source==tauArch){
				//setupMakefiles();
				//tcombo.select(0);
			}
			//System.out.println("ChangeText");
			updateLaunchConfigurationDialog();
		}
	}

	protected WidgetListener listener = new WidgetListener();

	private void checkMakefiles(){
		IPreferenceStore pstore = TAUOptionsPlugin.getDefault().getPreferenceStore();
		String archpath=pstore.getString("TAUCDTArchPath");
		String tlpath = archpath+File.separator+"lib";
		File taulib = new File(tlpath);
		
		class makefilter implements FilenameFilter{
			public boolean accept(File dir, String name) {
				if(name.indexOf("Makefile.tau")!=0 || name.indexOf("-pdt")<=0 || name.indexOf("-mpi")<=0 )
					return false;
				return true;
			}
		}
		if(taulib.exists()){
			makefiles = taulib.listFiles(new makefilter());
		}
		LinkedHashSet filenames = new LinkedHashSet();
		for(int i=0;i<makefiles.length;i++)
		{
			filenames.add(makefiles[i].getName());
		}
		
	}
	
	/**
	 * @see ILaunchConfigurationTab#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite comp = new Composite(parent, SWT.NONE);
		setControl(comp);

		GridLayout topLayout = new GridLayout();
		comp.setLayout(topLayout);

		Composite parallelComp = new Composite(comp, SWT.NONE);
		parallelComp.setLayout(createGridLayout(1, false, 0, 0));
		parallelComp.setLayoutData(spanGridData(GridData.FILL_HORIZONTAL, 5));

		createVerticalSpacer(parallelComp, 2);

		callpathCheck = createCheckButton(parallelComp, "Callpath Profiling");
		callpathCheck.addSelectionListener(listener);
		
		phaseCheck = createCheckButton(parallelComp, "Phase Based Profiling");
		phaseCheck.addSelectionListener(listener);
		
		//createVerticalSpacer(parallelComp, 2);
		memoryCheck = createCheckButton(parallelComp, "Memory Profiling");
		memoryCheck.addSelectionListener(listener);
		
		papiComp=new Composite(parallelComp,SWT.NONE);
		papiComp.setLayout(createGridLayout(2, false, 0, 0));
		papiComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		papiCheck = createCheckButton(papiComp, "PAPI");
		papiCheck.addSelectionListener(listener);
		//papiSelect=createPushButton(papiComp,"Set PAPI Environment Variables",null);
		//papiSelect.addSelectionListener(listener);
		
		perfCheck = createCheckButton(parallelComp, "Perflib");
		perfCheck.addSelectionListener(listener);
		
		traceCheck = createCheckButton(parallelComp, "Trace");
		traceCheck.addSelectionListener(listener);
		
		

		createVerticalSpacer(parallelComp, 2);
		selectLabel = new Label(parallelComp, SWT.SHADOW_NONE);
		selectLabel.setText("Selective Instrumentation");
		selectRadios = new Button[4];

		selectRadios[0]=createRadioButton(parallelComp, "None");
		selectRadios[1]=createRadioButton(parallelComp, "Internal");
		selectRadios[2]=createRadioButton(parallelComp, "User Defined");
		
		selComp = new Composite(parallelComp, SWT.NONE);		
		selComp.setLayout(createGridLayout(2, false, 0, 0));
		selComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		tauArch = new Text(selComp, SWT.BORDER | SWT.SINGLE);
		tauArch.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		tauArch.addModifyListener(listener);
		browseSelfileButton = createPushButton(selComp, "Browse", null);
		browseSelfileButton.addSelectionListener(listener);
		
		selectRadios[3]=createRadioButton(parallelComp, "Automatic");
		selectRadios[3].setEnabled(false);
		for(int i=0; i<selectRadios.length;i++)
			selectRadios[i].addSelectionListener(listener);

		createVerticalSpacer(parallelComp, 2);
		
		Composite comComp=new Composite(parallelComp,SWT.NONE);
		comComp.setLayout(createGridLayout(2, false, 0, 0));
		comComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Label compLab = new Label(comComp, 0);
		compLab.setText("Using Compiler:");
		compiler = new Text(comComp, SWT.BORDER | SWT.SINGLE);
		compiler.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		compiler.addModifyListener(listener);
		
		createVerticalSpacer(parallelComp, 2);
		
		nocleanCheck = createCheckButton(parallelComp, "Keep instrumented executable");
		nocleanCheck.addSelectionListener(listener);
		keepprofsCheck = createCheckButton(parallelComp, "Keep profiles");
		keepprofsCheck.addSelectionListener(listener);
		
	}

	public void updateComboFromSelection() {
		System.out.println("change startup");
	}

	/**
	 * Defaults are empty.
	 * 
	 * @see ILaunchConfigurationTab#setDefaults(ILaunchConfigurationWorkingCopy)
	 */
	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
		configuration.setAttribute(ITAULaunchConfigurationConstants.CALLPATH, ITAULaunchConfigurationConstants.CALLPATH_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.MEMORY, ITAULaunchConfigurationConstants.MEMORY_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.PAPI, ITAULaunchConfigurationConstants.PAPI_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.PERF, ITAULaunchConfigurationConstants.PERF_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.TRACE, ITAULaunchConfigurationConstants.TRACE_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.PHASE, ITAULaunchConfigurationConstants.PHASE_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.COMPILER, ITAULaunchConfigurationConstants.COMPILER_DEF);
		
		configuration.setAttribute(ITAULaunchConfigurationConstants.NOCLEAN, ITAULaunchConfigurationConstants.NOCLEAN_DEF);
		configuration.setAttribute(ITAULaunchConfigurationConstants.KEEPPROFS, ITAULaunchConfigurationConstants.KEEPPROFS_DEF);
		
		configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT, 0);
		configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT_FILE, "");
		
	}

	/**
	 * @see ILaunchConfigurationTab#initializeFrom(ILaunchConfiguration)
	 */
	public void initializeFrom(ILaunchConfiguration configuration) {
		try {
			callpathCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.CALLPATH, false));
			memoryCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.MEMORY, false));
			papiCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.PAPI, false));
			perfCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.PERF, false));
			traceCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.TRACE, false));
			phaseCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.PHASE, false));
			compiler.setText(configuration.getAttribute(ITAULaunchConfigurationConstants.COMPILER, ""));
			
			int selected = configuration.getAttribute(ITAULaunchConfigurationConstants.SELECT, 0);
			
			selectRadios[selected].setSelection(true);
			
			tauArch.setText(configuration.getAttribute(ITAULaunchConfigurationConstants.SELECT_FILE, ""));
			
			if(!selectRadios[2].getSelection()){
				selComp.setEnabled(false);
				tauArch.setEnabled(false);
				tauArch.setEnabled(false);
			}
			
			nocleanCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.NOCLEAN, false));
			keepprofsCheck.setSelection(configuration.getAttribute(ITAULaunchConfigurationConstants.KEEPPROFS, false));
			
		} catch (CoreException e) {
			setErrorMessage(CoreMessages
					.getFormattedResourceString(
							"CommonTab.common.Exception_occurred_reading_configuration_EXCEPTION",
							e.getStatus().getMessage()));
		}
		
	}

	/**
	 * @see ILaunchConfigurationTab#performApply(ILaunchConfigurationWorkingCopy)
	 */
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		configuration.setAttribute(ITAULaunchConfigurationConstants.CALLPATH, callpathCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.MEMORY, memoryCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.PAPI, papiCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.PERF, perfCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.TRACE, traceCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.PHASE, phaseCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.COMPILER, compiler.getText());
		
		configuration.setAttribute(ITAULaunchConfigurationConstants.NOCLEAN, nocleanCheck.getSelection());
		configuration.setAttribute(ITAULaunchConfigurationConstants.KEEPPROFS, keepprofsCheck.getSelection());
		
		int selected = 0;
		for(int i=0; i<selectRadios.length;i++)
		{
			if(selectRadios[i].getSelection())
			{
				selected = i;
				break;
			}
		}
		configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT, selected);
		
		configuration.setAttribute(ITAULaunchConfigurationConstants.SELECT_FILE, tauArch.getText());
	}


	protected String getFieldContent(IntegerFieldEditor editorField) {
		return getFieldContent(editorField.getStringValue());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#isValid(org.eclipse.debug.core.ILaunchConfiguration)
	 */
	public boolean isValid(ILaunchConfiguration configuration) {
		setErrorMessage(null);
		setMessage(null);

		//if (!numberOfProcessField.isValid()) {
		//	setErrorMessage(numberOfProcessField.getErrorMessage());
		//	return false;
		//}

		/*
		 * if (getFieldContent(numberOfProcessStartField) != null &&
		 * !numberOfProcessStartField.isValid()) {
		 * setErrorMessage(numberOfProcessStartField.getErrorMessage()); return
		 * false; }
		 * 
		 * if (getFieldContent(firstNodeNumberField) != null &&
		 * !firstNodeNumberField.isValid()) {
		 * setErrorMessage(firstNodeNumberField.getErrorMessage()); return
		 * false; }
		 */

		return true;
	}

	protected void handleSelfileBrowseButtonSelected() 
	{
		FileDialog dialog = new FileDialog(getShell());
		dialog.setText("Select TAU Selective Instrumentation File");
		
		String correctPath = getFieldContent(tauArch.getText());
		if (correctPath != null) {
			File path = new File(correctPath);
			if (path.exists())
				dialog.setFilterPath(path.isFile() ? correctPath : path
						.getParent());
		}

		String selectedPath = dialog.open();
		if (selectedPath != null)
			tauArch.setText(selectedPath);
	}
	
	 public class ListDialog extends Dialog { 
		 Object result; 
		 public ListDialog (Shell parent, int style){ 
			 super (parent, style); } 
		 	public ListDialog (Shell parent)
		 	{ 
		 		this (parent, 0); // your default style bits go here (not the Shell's style bits) 
		 	} 
		 	public Object open(){ 
		 		Shell parent = getParent(); 
		 		Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL); 
		 		shell.setText(getText()); 
		 		// Your code goes here (widget creation, set result, etc). 
		 		//shell.setSize(100, 400);
		 		shell.setLayout(createGridLayout(1, false, 0, 0));
				shell.setLayoutData(spanGridData(GridData.FILL_HORIZONTAL, 5));
		 		List papilist = new List(shell, SWT.MULTI);
		 		
		 		Composite endcomp=new Composite(shell,SWT.NONE);
		 		endcomp.setLayout(createGridLayout(2, false, 0, 0));
		 		endcomp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		 		
		 		Button okay = createPushButton(endcomp, "Okay", null);
		 		Button cancel = createPushButton(endcomp, "Cancel", null);
		 		papilist.add("yesy");
		 		papilist.add("nowu");
		 		papilist.add("mayb");
		 		papilist.pack();
		 		okay.pack();
		 		cancel.pack();
		 		shell.pack();
		 		shell.open(); 
		 		Display display = parent.getDisplay(); 
		 		while (!shell.isDisposed()){ 
		 			if (!display.readAndDispatch()) display.sleep(); 
		 		} 
		 		return result; } }
	
	protected void handlePapiSelect() 
	{
		//org.eclipse.swt.widgets.
		
		ListDialog papidialog = new ListDialog(getShell());
		
		//FileDialog dialog = new FileDialog(getShell());
		papidialog.setText("Select PAPI Counters");
		papidialog.open();
		/*
		String correctPath = getFieldContent(tauArch.getText());
		if (correctPath != null) {
			File path = new File(correctPath);
			if (path.exists())
				dialog.setFilterPath(path.isFile() ? correctPath : path
						.getParent());
		}

		String selectedPath = papidialog.open();
		if (selectedPath != null)
			tauArch.setText(selectedPath);*/
	}
	
	/**
	 * @see ILaunchConfigurationTab#getName()
	 */
	public String getName() {
		return "Analysis";//CoreMessages.getResourceString("ParallelTab.Parallel");
	}

	/**
	 * @see ILaunchConfigurationTab#setLaunchConfigurationDialog(ILaunchConfigurationDialog)
	 */
	public void setLaunchConfigurationDialog(ILaunchConfigurationDialog dialog) {
		super.setLaunchConfigurationDialog(dialog);
	}

	/**
	 * @see ILaunchConfigurationTab#getImage()
	 */
	public Image getImage() {
		return LaunchImages.getImage(LaunchImages.IMG_ANALYSIS_TAB);
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}